@@ -36,8 +36,6 @@
 #include <libxml/parser.h>
 #include <libxml/tree.h>
 
-#include <gconf/gconf-client.h>
-
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
 #include <glib-unix.h>
@@ -64,6 +62,10 @@
 #define OPENCONNECT_OPENSSL
 #endif
 
+#if !OPENCONNECT_CHECK_VER(2,1)
+#define openconnect_set_stoken_mode(...) -EOPNOTSUPP
+#endif
+
 #ifdef OPENCONNECT_OPENSSL
 #include <openssl/ssl.h>
 #include <openssl/bio.h>
@@ -101,12 +103,6 @@ enum certificate_response{
 	CERT_ACCEPTED,
 };
 
-struct gconf_key {
-	char *key;
-	char *value;
-	struct gconf_key *next;
-};
-
 /* This struct holds all information we need to add a password to
  * gnome-keyring. It’s used in success_passwords. */
 struct keyring_password {
@@ -1043,6 +1039,8 @@ static int get_config (GHashTable *options, GHashTable *secrets,
 	char *csd_wrapper;
 	char *pem_passphrase_fsid;
 	char *cafile;
+	char *stoken_source;
+	char *stoken_string;
 
 	hostname = g_hash_table_lookup (options, NM_OPENCONNECT_KEY_GATEWAY);
 	if (!hostname) {
@@ -1089,17 +1087,18 @@ static int get_config (GHashTable *options, GHashTable *secrets,
 	if (cafile)
 		openconnect_set_cafile(vpninfo, g_strdup (cafile));
 
-	csd = g_hash_table_lookup (options, "enable_csd_trojan");
+	csd = g_hash_table_lookup (options, NM_OPENCONNECT_KEY_CSD_ENABLE);
 	if (csd && !strcmp(csd, "yes")) {
 		/* We're not running as root; we can't setuid(). */
-		csd_wrapper = g_hash_table_lookup (options, "csd_wrapper");
+		csd_wrapper = g_hash_table_lookup (options,
+						   NM_OPENCONNECT_KEY_CSD_WRAPPER);
 		if (csd_wrapper && !csd_wrapper[0])
 			csd_wrapper = NULL;
 
 		openconnect_setup_csd(vpninfo, getuid(), 1, g_strdup (csd_wrapper));
 	}
 
-	proxy = g_hash_table_lookup (options, "proxy");
+	proxy = g_hash_table_lookup (options, NM_OPENCONNECT_KEY_PROXY);
 	if (proxy && proxy[0] && openconnect_set_http_proxy(vpninfo, g_strdup (proxy)))
 		return -EINVAL;
 
@@ -1107,10 +1106,25 @@ static int get_config (GHashTable *options, GHashTable *secrets,
 	sslkey = g_hash_table_lookup (options, NM_OPENCONNECT_KEY_PRIVKEY);
 	openconnect_set_client_cert (vpninfo, g_strdup (cert), g_strdup (sslkey));
 
-	pem_passphrase_fsid = g_hash_table_lookup (options, "pem_passphrase_fsid");
+	pem_passphrase_fsid = g_hash_table_lookup (options,
+						   NM_OPENCONNECT_KEY_PEM_PASSPHRASE_FSID);
 	if (pem_passphrase_fsid && cert && !strcmp(pem_passphrase_fsid, "yes"))
 		openconnect_passphrase_from_fsid(vpninfo);
 
+	stoken_source = g_hash_table_lookup (options, NM_OPENCONNECT_KEY_STOKEN_SOURCE);
+	stoken_string = g_hash_table_lookup (options, NM_OPENCONNECT_KEY_STOKEN_STRING);
+	if (stoken_source) {
+		int ret = 0;
+
+		if (!strcmp(stoken_source, "manual") && stoken_string)
+			ret = openconnect_set_stoken_mode(vpninfo, 1, stoken_string);
+		else if (!strcmp(stoken_source, "stokenrc"))
+			ret = openconnect_set_stoken_mode(vpninfo, 1, NULL);
+
+		if (ret)
+			fprintf(stderr, "Failed to initialize stoken: %d\n", ret);
+	}
+
 	return 0;
 }
 
@@ -1506,14 +1520,14 @@ static void build_main_dialog(auth_ui_data *ui_data)
 	g_signal_connect_swapped(ui_data->connect_button, "clicked",
 				 G_CALLBACK(queue_connect_host), ui_data);
 	gtk_widget_show(ui_data->connect_button);
-
-	autocon = gtk_check_button_new_with_label(_("Automatically start connecting next time"));
-	gtk_box_pack_start(GTK_BOX(vbox), autocon, FALSE, FALSE, 0);
-	if (get_autoconnect (ui_data->secrets))
-		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(autocon), 1);
-	g_signal_connect(autocon, "toggled", G_CALLBACK(autocon_toggled), NULL);
-	gtk_widget_show(autocon);
-
+	if (vpnhosts->next) {
+		autocon = gtk_check_button_new_with_label(_("Automatically start connecting next time"));
+		gtk_box_pack_start(GTK_BOX(vbox), autocon, FALSE, FALSE, 0);
+		if (get_autoconnect (ui_data->secrets))
+			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(autocon), 1);
+		g_signal_connect(autocon, "toggled", G_CALLBACK(autocon_toggled), NULL);
+		gtk_widget_show(autocon);
+	}
 	frame = gtk_frame_new(NULL);
 	gtk_box_pack_start(GTK_BOX(vbox), frame, TRUE, TRUE, 0);
 	gtk_widget_set_size_request(frame, -1, -1);
@@ -1762,7 +1776,7 @@ int main (int argc, char **argv)
 
 	_ui_data = init_ui_data(vpn_name, options, secrets, vpn_uuid);
 	if (get_config(options, secrets, _ui_data->vpninfo)) {
-		fprintf(stderr, "Failed to find VPN UUID %s in gconf\n", vpn_uuid);
+		fprintf(stderr, "Failed to find VPN UUID %s\n", vpn_uuid);
 		return 1;
 	}
 	build_main_dialog(_ui_data);
@@ -1772,7 +1786,8 @@ int main (int argc, char **argv)
 #endif
 	openconnect_init_ssl();
 
-	if (get_autoconnect (secrets))
+	/* Start connecting now if there's only one host. Or if configured to */
+	if (!vpnhosts->next || get_autoconnect (secrets))
 		queue_connect_host(_ui_data);
 
 	gtk_window_present(GTK_WINDOW(_ui_data->dialog));
